require 'BoundingBox'

EntityController = class('EntityController', nil)
EntityController.entities = {}

function EntityController.updateInstances( dt )
    for k, entity in ipairs(EntityController.entities) do
        entity:update( dt )
    end
end

function EntityController.getEntitiesByClass( ... )
    local classes = {...}
    local selectedEntities = {}
    
    while #classes > 0 do
        local class = classes[#classes]
        for i, entity in ipairs(EntityController.entities) do
            if instanceOf( class, entity ) then
                selectedEntities[#selectedEntities+1] = entity
            end
        end
        classes[#classes] = nil
    end
    
    return selectedEntities
end

function EntityController.getEntities()
    return EntityController.entities
end

function EntityController:initialize()
    table.insert( EntityController.entities, self )
    self.position = vector(0,0)
    self.boundingBox = BoundingBox()
end

function EntityController:destroy()
    for i, entity in ipairs(EntityController.entities) do
        if entity == self then
            table.remove( EntityController.entities, i )
            break
        end
    end
end

function EntityController:update( dt )
    -- stub function
end

function EntityController:className()
    return self.class.name
end

function EntityController:getPosition()
    return self.position:clone()
end

function EntityController:getBoundingBox()
    return self.boundingBox
end