Hexagon = class('Hexagon', nil)
Hexagon.sin60 = math.sin( math.rad( 60 ) )
Hexagon.sin30 = math.sin( math.rad( 30 ) )
Hexagon.cos30 = math.cos( math.rad( 30 ) )

function Hexagon:initialize( length )
	local h, r
	h = self.sin30 * length
	r = self.cos30 * length
	self.length = length
	self.h = h
	self.r = r
	self.height = 2 * r
	self.width  = length + 2 * h
	self.horizontalOffset = length + h
	self.verticalOffset   = self.r 
	self.aa = vector( -h, 0 )
	self.bb = vector( length + h, self.height )
	self.vertices = {
		0, 0,
		length, 0,      --N
		length + h, r,  --NW
		length, r + r,  --SW
		0, r + r,       --S
		-h, r           --SE
		                --NE (0,0)
	}
end

function Hexagon:getEdgeBySide( side, inset )
    local edge = {}
    local start = side * 2
    for i = start, start + 3 do
        local i1 = (i % 12) + 1
        edge[#edge + 1] = self.vertices[ i1 ]
    end
    
    if inset then
        local normalize = vector.normalize
        local cx, cy = self:getCenter()
        local vec1 = vector( cx, cy )
        local vec2 = vector( edge[1], edge[2] )
        local vec3 = vector( edge[3], edge[4] )
        local vec1_2 = normalize( vec1 - vec2 )
        local vec1_3 = normalize( vec1 - vec3 )
        vec2 = vec2 + vec1_2 * inset
        vec3 = vec3 + vec1_3 * inset
        edge[1] = vec2.x
        edge[2] = vec2.y
        edge[3] = vec3.x
        edge[4] = vec3.y
    end
    
    return edge
end

function Hexagon:getCenter()
    return (self.length / 2), self.r
end

function Hexagon:isPointInside( px, py )
    -- transform point to always be in bottom right quadrant.
    local cx, cy = self:getCenter()
    local qx = math.abs( px - cx ) + cx
    local qy = math.abs( py - cy ) + cy
    -- check bounds.
    if qx > cx+self.length or qy > self.height then return false end
    -- check corner.
    local p = vector(self.vertices[7], self.vertices[8])
    local d = -p.y * (qx - p.x) - p.x * (qy - p.y)
    return d >= 0
end

function Hexagon:render( x, y )
	love.graphics.push()
	if shouldOffset then
	    love.graphics.translate( x, y )
    else
	    love.graphics.translate( x, y )
    end
	---
	love.graphics.polygon( "fill", self.vertices )
	love.graphics.setColor(255,0,0)
	local cx, cy = self:getCenter()
	--love.graphics.circle( "line", cx, cy, 5 )
	--love.graphics.circle( "line", 0, 0, 5 )
	---
	love.graphics.pop()
end

function Hexagon:drawGrid( x, y, width, height, spacing )
	local spacing = spacing or 1
	local a = self.h + self.length
	local b = self.r
	love.graphics.push()
	love.graphics.translate( x, y )
	--
	for j = 0, height do
	for i = 0, width do
		if i % 2 == 0 then
			--even
			self:render( a * i * spacing, self.height * j * spacing )
		else
			--odd
			self:render( a * i * spacing, (self.height * j * spacing) + (b * spacing))
		end
	end
	end
	--
	love.graphics.pop()
end