InputController = class('InputController', nil)

MOUSE_OFF       = 0
MOUSE_OVER      = 1
MOUSE_DOWN      = 2

function InputController:initialize()
    self.mouseOverState = MOUSE_OFF
    local tags = { "input", self }
    NotificationController.listen("on_mouse_button_pressed", tags, self.onMouseButtonPressed, self)
end

function InputController:update(dt)
    self:updateMouseDownState( getCurrentCamera():mousepos(), 'l', 'r', 'm' )
end

function InputController:updateMouseDownState( mouseVec, boundingBox, ... )
    if boundingBox:isPointInside( mouseVec.x, mouseVec.y ) then
        if love.mouse.isDown( unpack({...}) ) then
            if self.mouseOverState ~= MOUSE_DOWN then self:onEnterMouseDown( ) end
            self.mouseOverState = MOUSE_DOWN
        else
            if self.mouseOverState ~= MOUSE_OVER then self:onEnterMouseOver( ) end
            self.mouseOverState = MOUSE_OVER
        end
    else
        if self.mouseOverState ~= MOUSE_OFF then self:onEnterMouseOff( ) end
        self.mouseOverState = MOUSE_OFF
    end
end

function InputController:getMouseOverState()
    return self.mouseOverState
end

function InputController:onMouseButtonPressed( mx, my, button )
end

function InputController:onEnterMouseOff( )
end      
         
function InputController:onEnterMouseOver( )
end      
         
function InputController:onEnterMouseDown( )
end

function InputController:getEntityUnderMouse( ... )
    local entities = EntityController.getEntitiesByClass( unpack({...}) )
    for i, entity in ipairs(entities) do
        local mouseOverState = entity.inputController:getMouseOverState()
        if  mouseOverState ~= MOUSE_OFF then
            return entity
        end
    end
    
    return false
end

function InputController:halt()
    NotificationController.muteTags( {self} )
    self._update = self.update
    self.update = function() end
end

function InputController:resume()
    NotificationController.unmuteTags( {self} )
    self.update = self._update
end