NotificationController = {}
NotificationController.listeners = {}
NotificationController.currentID = 0
NotificationController.listenerHasTags = {}
TAGS_ANY  = 1
TAGS_ALL  = 2
TAGS_ONLY = 3
local this = NotificationController


function NotificationController.listen( name, tags, callback, ... )
	if this.listeners[name] == nil then
		this.listeners[name] = {}
	end
	if tags and type(tags) ~= "table" then 
	    tags = {tags}
	end
	local id = this.getUniqueID()
	local listener = {
		['id'] 			= id,
		['name']		= name,
		['tags'] 		= tags,
		['callback'] 	= callback,
		['args'] 		= {...},
		['active'] 		= true
	}
	table.insert( this.listeners[name], listener )
	return listener
end

function NotificationController.send( name, tags, ... )
	local args = {...}
	if tags and type(tags) ~= "table" then 
	    tags = {tags}
	end
	local listeners = this.listeners[name]
	if listeners then
		for i, listener in ipairs(listeners) do
			if listener.active and (tags == nil or this.listenerHasTags[TAGS_ANY]( listener, tags )) then
				listener.callback( unpack(listener.args), unpack(args) )
			end
		end
	end
end

function NotificationController.muteListener( listener )
	listener.active = false
end

function NotificationController.unmuteListener( listener )
	listener.active = true
end

function NotificationController.killListener( listener )
	for i, v in ipairs(this.listeners[ listener.name ]) do
		if v.id == listener.id then
			table.remove( this.listeners[ listener.name ], i)
			break
		end
	end
	listener = nil
end

function NotificationController.muteTags( tags, checkType )
	local checkType = checkType or TAGS_ANY
	for _, listenerType in pairs(this.listeners) do
		for _, listener in ipairs(listenerType) do
			if this.listenerHasTags[checkType]( listener, tags ) then
				this.muteListener( listener )
			end
		end
	end
end

function NotificationController.unmuteTags( tags, checkType )
	local checkType = checkType or TAGS_ANY
	for _, listenerType in pairs(this.listeners) do
		for _, listener in ipairs(listenerType) do
			if this.listenerHasTags[checkType]( listener, tags ) then
				this.unmuteListener( listener )
			end
		end
	end
end

function NotificationController.getUniqueID()
	this.currentID = this.currentID + 1
	return this.currentID
end

NotificationController.listenerHasTags[TAGS_ANY] = function( listener, tags )
    if listener.tags then
    	for _, listenerTag in ipairs(listener.tags) do
    		for _, testingTag in ipairs( tags ) do
    			if listenerTag == testingTag then return true end
    		end
    	end
	end
	return false
end

NotificationController.listenerHasTags[TAGS_ALL] = function( listener, tags )
	local amount = #tags
	
	for _, listenerTag in ipairs(listener.tags) do
		for _, testingTag in ipairs( tags ) do
			if listenerTag == testingTag then return true end
		end
	end
	
	return false
end