require 'InputController'

ProgramInputController = class('ProgramInputController', InputController)

function ProgramInputController:initialize()
    InputController.initialize(self)
end

function ProgramInputController:setProgramController( programController )
    self.programController = programController
end

function ProgramInputController:update( dt )
    if GridController.instance:getSystemUnderMouse() == self.programController.systemController then
        local p = getCurrentCamera():localMouseCoords( self.programController.systemController.position )
        local boundingBox = self.programController:getBoundingBox()
        self:updateMouseDownState( p, boundingBox, 'l', 'm', 'r' )
    else
        self.mouseOverState = MOUSE_OFF
    end
end

function ProgramInputController:onMouseButtonPressed( mx, my, button )
    local p = getCurrentCamera():localMouseCoords( self.programController.systemController.position )
    if self.programController:getSelected() and self.mouseOverState == MOUSE_OFF then
        if GridController.instance:isSystemUnderMouse( self.programController.systemController ) then
            local entityUnderMouse = self:getEntityUnderMouse( ProgramController, PortController )
            if button == 'r' then 
                if not entityUnderMouse then
                    self.programController:addMoveOrder( p.x, p.y )
                elseif instanceOf( PortController, entityUnderMouse ) then
                    self.programController:addPortOrder( entityUnderMouse )
                elseif self.programController:isEnemyWith( entityUnderMouse ) then
                    self.programController:addAttackOrder( entityUnderMouse )
                elseif self.programController:isAlliedWith( entityUnderMouse ) then
                    self.programController:addAssistOrder( entityUnderMouse )
                end
            end
            if button == 'l' then
                if not programUnderMouse then
                    NotificationController.send("deselect_all_programs")
                end
            end
        else
            if button == 'r' then
                local systemUnderMouse = GridController.instance:getSystemUnderMouse()
                if systemUnderMouse then
                    self.programController:navigateToSystem( systemUnderMouse )
                end
            end
        end
    end
end

function ProgramInputController:onMouseButtonReleased( mx, my, button )
end

function ProgramInputController:onEnterMouseOff( )
end

function ProgramInputController:onEnterMouseOver( )
end

function ProgramInputController:onEnterMouseDown( )
    if love.mouse.isDown('l') then
        self.programController:setSelected( true, true )
    end
end

