require 'specs.init'
require 'PortController'
require 'Hexagon'

describe["A Port"] = function()
    before = function()
        MockSystemControllerA = Mock:new()
        MockSystemControllerA.numberOfPortsOnSide:returns( 1 )
        MockSystemControllerA.hexagonShape = Hexagon( 100 )
        MockSystemControllerB = Mock:new()
        MockSystemControllerB.numberOfPortsOnSide:returns( 1 )
        MockSystemControllerB.hexagonShape = Hexagon( 100 )
        MockPortRenderer = Mock:new()
        MockProgramController = Mock:new()
        portA = PortController( MockSystemControllerA, DIR_N, MockPortRenderer )
        portB = PortController( MockSystemControllerB, DIR_S, MockPortRenderer )
        portA:connectToPort( portB )
    end
    
    after = function()
        portA.packets = nil
        portB.packets = nil
    end
    
    it["should have a sister port"] = function()
        expect( portA:getSisterPort() ).should_be( portB )
    end
    
    describe["when a program enters"] = function()
        before = function()
            portA:transferProgram( MockProgramController )
        end
    
        it["should have a reference to the program"] = function()
            expect( portA:hasProgram( MockProgramController ) ).should_be( true )
        end
    end
    
    describe["when multiple programs enter"] = function()
        before = function()
            program1 = Mock:new()
            program2 = Mock:new()
            portA:transferProgram( program1 )
            portA:transferProgram( program2 )
        end
        
        it["should have a single packet containing references to the programs"] = function()
            local packet1 = portA:getPacketWithProgram( program1 )
            local packet2 = portA:getPacketWithProgram( program2 )
            expect( packet1 ).should_be( packet2 )
        end
    end

    
    describe["when packet transfer complete"] = function()
        before = function()
            program1 = Mock:new()
            portA:transferProgram( program1 )
            packet = portA:getPacketWithProgram( program1 )
            portA:deliverPacket( packet )
        end
        
        it["should remove the packet"] = function()
            expect( portA:hasPacket( packet ) ).should_be( false )
        end
        
        it["should add the programs inside the packet to the sister sytem"] = function()
            local calls = #Mock.calls[ MockSystemControllerB.addProgram ]
            local call_1_argument_2 = Mock.calls[ MockSystemControllerB.addProgram ][1][2]
            expect( calls ).should_be( 1 )
            expect( call_1_argument_2 ).should_be( program1 )
        end
    end
end